(* File main.ml *)
(* Stephanie Weirich, adapted from Amr Sabry *)

module P  = EoplParser
module L  = EoplLexer
module PP = EoplPrint
module E  = Eval
module T  = Typecheck

open Format

let _ = 
  try 
    flush stdout;
    let lexbuf = Lexing.from_channel stdin in

      while true do
        let f _ : unit = 
          try
            let e = P.main L.token lexbuf in
              print_string "Input expression: ";
              print_newline();
              
              PP.print_exp e;
              print_newline ();

				  print_string "Type:";
				  print_string (PP.string_of_typ (T.type_program e));
				  print_newline ()

              (* E.eval_program e *)

          with 
              Parsing.Parse_error ->
                (L.print_error "Parse error\n")
            | T.TODO -> 
                (print_string "Not done\n")
            | E.RuntimeError s -> 
                (print_string s; print_newline ())
				| T.TypeError s -> 
					 (print_string "Type error:"; print_string s; print_newline ())
				| Env.Error s  -> 
					 (print_string s; print_newline ())
        in
          
          f ();
          print_string "\n******************\n";
          flush stdout
      done
  with L.Eof -> exit 0


