/* File mparser.mly --- the lambda calculus */
/* Stephanie Weirich, adapted from code by Amr Sabry */

%token LAMBDA LPAREN RPAREN PLUS TIMES MINUS LET
%token <string>ID
%token <int> NUMBER
%token TRUE FALSE
%token IF EQUAL AND OR NOT

%start main
%type <Absyn.exp> main

%%

main: 
          exp { $1 }
;

exp: 
          ID { Absyn.Var ($1) }
        | LPAREN pexp RPAREN { ($2) }
		  | NUMBER { Absyn.LitInt ($1) }
		  | TRUE   { Absyn.LitBool (true) }
		  | FALSE  { Absyn.LitBool (false) }
;

pexp :  
          exp exps { Absyn.App ($1,$2) }
		  | prim exps { Absyn.Prim ($1,$2) }
		  | LET LPAREN binds RPAREN exp
				{ Absyn.Let (fst $3,snd $3,$5) }
		  | IF exp exp exp 
				{ Absyn.If ($2,$3,$4) }
        | LAMBDA LPAREN ids RPAREN exp 
				{ Absyn.Lambda ($3, $5) }
;

ids:     { [] }
		  | ID ids { ($1::$2) }
;

binds:   { [],[] }
	     | LPAREN ID exp RPAREN binds { ($2::fst $5,$3::snd $5) }
; 

prim:     PLUS  { Absyn.Plus  }
        | TIMES { Absyn.Times }
        | MINUS { Absyn.Minus }
		  | EQUAL { Absyn.Equal }
		  | AND   { Absyn.And   }
		  | OR    { Absyn.Or    }
		  | NOT   { Absyn.Not   }
;

exps:   /* empty */ { [] } 
        | exp exps  { ($1 :: $2) }
;
%%

