open Absyn 
open Env

exception TypeError of string
exception TODO

type tenv = typ Env.env

(* Apply the function f to all pairs of elements in f1 f2 and return true if 
	all results are true *)
let all2 f l1 l2 : bool = List.fold_left2 (fun x y z -> x && f y z) true l1 l2

(* Type check a primitive, given a list of arguments. *)
let rec check_prim (p:prim) (args:typ list) : typ = 
  match p,args with 
      Plus,   [IntType;IntType] -> IntType 
    | Times,  [IntType;IntType] -> IntType
    | Minus,  [IntType;IntType] -> IntType
    | And,    [BoolType; BoolType] -> BoolType 
    | Or,     [BoolType; BoolType] -> BoolType 
    | Not,    [BoolType] -> BoolType 
    | Add1,   [IntType] -> IntType 
    | Sub1,   [IntType] -> IntType 
    | Zero,   [IntType] -> BoolType
    | LT,     [IntType; IntType] -> BoolType      
    | GT,     [IntType; IntType] -> BoolType
    | Equal,  [IntType;IntType] -> BoolType

	 (* For II.2 *) 
    | Pair,   _ -> raise TODO
    | Fst,    _ -> raise TODO
    | Snd,    _ -> raise TODO
		  
	 (* For II.3 *)
    | Hd,     _ -> raise TODO
    | Tl,     _ -> raise TODO
    | Cons,   _ -> raise TODO
    | IsNull, _ -> raise TODO

    | _, _ -> raise (TypeError "Unimplemented  primitive or invalid arguments")

(* Extend the type environment for a letrec expression and check that 
	all of the rhs's are well-typed. *)
let rec extend_env_letrec 
  (tys:typ list)
  (proc_names:var list)
  (arg_typs: typ list list) 
  (idss: var list list) 
  (bodies: exp list)
  (env: tenv) : tenv = raise TODO
      
and type_exp (exp:exp) (env:tenv) : typ = 
  match exp with 
      LitInt i -> IntType
    | LitBool b -> BoolType
    | LitNull t -> raise TODO
    | Var v -> apply_env env v
    | Prim (p, rands) -> 
        let args = type_exp_rands rands env in
          check_prim p args
    | If (test_exp, true_exp, false_exp) -> raise TODO
    | Let (vars,rands,body) -> raise TODO
    | Lambda (tys,ids,body) ->    
        ProcType (tys, type_exp body (extend_env ids tys env))
    | App (rator,rands) -> 
        let proc = type_exp rator env in 
        let args = type_exp_rands rands env in 
          (match proc with 
               ProcType (arg_typs, ret) -> 
                 (try 
                    if all2 (fun t1 t2 -> t1 = t2) args arg_typs then ret
                    else raise (TypeError "Invalid argument to function")
                  with 
                      Invalid_argument _ -> raise (TypeError "Wrong number of args to function"))
             | _ -> raise (TypeError "Attempt to apply non-function"))
    | Letrec (tys, proc_names, arg_tys, idss, bodies, letrec_body) -> 
		  raise TODO
    | Varassign (id, rhs) -> raise TODO
    | Begin(exp, exps) -> 
        let v = type_exp exp env in
        let rec loop l = 
          match l with 
              [] -> v
            | [e] -> type_exp e env 
            | (hd :: tl) -> 
                ignore (type_exp hd env) ;
                loop tl in 
          loop exps
            
and type_exp_rands (rands:exp list) (env:tenv): typ list = 
  List.map (fun x -> type_exp x env) rands

let type_program (e:exp) : typ = type_exp e Empty
